<?php
namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;

class ArquivosController extends Crud {

    protected $dataSourceGetAnexos = 'gpeAnexos_getAnexos';
    protected $dataSourceGetAnexosDocParc = 'gpeAnexos_getAnexosDocParc';
    protected $dataSourceGetFotoOperadorBTL = 'gpeAnexos_getFotoOperadorBTL';
    protected $dataSourceGetDocModelo = 'gpeAnexos_getDocModelo';
    protected $dataSourceGetDocModeloParceiro = 'gpeAnexos_getDocModeloParceiro';
    
    protected $parameterBagColumns = array('NRORGPADRAO', 'IDFOTO', 'NRPARCNEGOCIO', 'NRORG', 'P_NRPARCNEGOCIO_OPERADOR','DEFAULT_PHOTO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $HCMParameters) {
        $this->parameterBag = $parameterBag;
        $this->HCMParameters = $HCMParameters;
        parent::__construct($dataSourceManager);
    }
    
    public function buscarFoto (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $defaultPhoto = null;
            $filterData->setDataSourceName($this->dataSourceGetAnexos); 
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            foreach($filterData->getConditions() as $conditions){
                $defaultPhoto = $conditions['columnName'] === 'DEFAULT_PHOTO';
            }
            if($defaultPhoto){
                if(empty($dataSet->getRows())){
                    $dataSet = $this->getFotoDefault($request, $response);
                }
            }
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getFotoDefault (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetFotoOperadorBTL);
            $filterData->addCondition('P_NRPARCNEGOCIO_OPERADOR', 5);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            return $dataSet;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscarArquivos (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetAnexos);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function buscarArquivosDoc (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetAnexosDocParc);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*Documento Modelo (Contratos/Termos)*/
    public function getDocModelo(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetDocModelo);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getDocModeloParceiro(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetDocModeloParceiro);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function baixarArquivo (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            
            if(array_key_exists('CMARQUIVO', $request->getDataSet()->getRows()[0]))
                $url = $request->getDataSet()->getRows()[0]['CMARQUIVO'];

            if(array_key_exists('CAMINHOFOTO', $request->getDataSet()->getRows()[0]))
                $url = $request->getDataSet()->getRows()[0]['CAMINHOFOTO'];

            $explode = explode('/', $url);
            if(strpos($explode[count($explode)-1], FPA::getTempDirectory()) === false){
                $arquivo = FPA::getTempDirectory().DIRECTORY_SEPARATOR.$explode[count($explode)-1];
            }else{
                $arquivo = $explode[count($explode)-1];
            }
            
            file_put_contents($arquivo, file_get_contents($url));
            if(isset($arquivo) && file_exists($arquivo)){
                $tipo = null;
                switch(strtolower(substr(strrchr(basename($arquivo),"."),1))){
                 case "pdf": $tipo="application/pdf"; break;
                 case "exe": $tipo="application/octet-stream"; break;
                 case "zip": $tipo="application/zip"; break;
                 case "doc": $tipo="application/msword"; break;
                 case "xls": $tipo="application/vnd.ms-excel"; break;
                 case "ppt": $tipo="application/vnd.ms-powerpoint"; break;
                 case "gif": $tipo="image/gif"; break;
                 case "png": $tipo="image/png"; break;
                 case "jpg": $tipo="image/jpg"; break;
                 case "mp3": $tipo="audio/mpeg"; break;
                 case "mp4": $tipo="video/mpeg-4"; break;
                 case "wma": $tipo="audio/mime"; break;
                 case "txt": $tipo="text/plain"; break;
                 case "php": // deixar vazio por segurança
                 case "htm": // deixar vazio por segurança
                 case "html": // deixar vazio por segurança
                }

                header("Content-Type: ".$tipo); // informa o tipo do arquivo ao navegador
                header("Content-Length: ".filesize($arquivo)); // informa o tamanho do arquivo ao navegador
                header("Content-Disposition: attachment; filename=".(substr(basename($arquivo),0))); // informa ao navegador que é tipo anexo e faz abrir a janela de download, tambem informa o nome do arquivo
                readfile($arquivo); // lê o arquivo
                unlink($arquivo);
                die();
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    function base64_encode_image($filename, $filetype) {
        if ($filename) {
            $imgbinary = file_get_contents($filename);
            return 'data:image/' . $filetype . ';base64,' . base64_encode($imgbinary);
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

}