<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class ComentariosController extends Crud {

    protected $dataSourceName  = 'comentarios';
    protected $dataSourceName2 = 'comentarios_candidatos';
    protected $dataSourceName3 = 'comentarios_save';
    protected $dataSourceSetIdParecerFinal = 'desParticipaProcesso_setIdParecerFinal';
    protected $parameterBagColumns = array('NRORG' , 'NRPARCNEGOCIO', 'NRPROCESSO', 'NRFASE');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function buscarDadosCandidatos (DTO\Request\Filter $request, DTO\Response $response) {
        unset($this->parameterBagColumns[2]);
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarComentario (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $aux = $this->dataSourceName;
            $this->dataSourceName = $this->dataSourceName3;
            foreach($request->getDataSet()->getRows() as &$row) {
                if ($row['__is_new'] === true){
                    $row['NRCOMENTAPROCESSO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_COMENTAPROCESSO', 1, 12, true);
                    $__is_new = true;
                } else {
                    $__is_new = false;
                }

                if(($row['IDTIPOCOMENTARIO'] == 'COMENTARIO') && ($row['DELETEPARECERFINAL']) || ($row['IDTIPOCOMENTARIO'] == 'LAUDO')){
                    $row['__is_new'] = false;
                    $this->dataSourceName = $this->dataSourceSetIdParecerFinal;
                    if(array_key_exists('CDOPERRESPONSAVEL', $row)){
                        $row['CDOPERULTATU'] = $row['CDOPERRESPONSAVEL'];
                    }

                    if(array_key_exists('NRORG', $row)){
                        $row['NRORGULTATU'] = $row['NRORG'];
                    }

                    if(!array_key_exists('DTULTATU', $row)){
                        $row['DTULTATU'] = date('d/m/Y');
                    }
                    parent::save($request, $response);
                    $this->dataSourceName = $this->dataSourceName3;
                    $row['__is_new'] = $__is_new;
                }
                        
                if(!array_key_exists('DSENTPSICOLOGICA', $row)){
                    $row['DSENTPSICOLOGICA'] = null;
                }
                        
                if(!array_key_exists('DSRESULTESTES', $row)){
                    $row['DSRESULTESTES'] = null;
                }

                if(!array_key_exists('NRFASE', $row)){
                    $row['NRFASE'] = null;
                }
                parent::save($request, $response);
            }
            $this->dataSourceName = $aux;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName3);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}