<?php

namespace HCM\FPA\Controllers;

use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class PensaoController extends Crud {

    protected $dataSourceNamePensao_get = 'fpa_pensao_get';
    protected $dataSourceNameVinculoPensao_get = 'gpe_vinculomPensao_get';
    protected $parameterBagColumns = array('P_NRORG','P_DTMESCOMPETENC','P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }
    
    public function getPensao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNamePensao_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getVinculosPensao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameVinculoPensao_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}