<?php

namespace HCM\FPA\Registro\grrf;

use HCM\FPA\Registro\RegistroGRRF;
use HCM\Util\DateUtil;

class RegistroInfComplGRRF extends RegistroGRRF{

    private $_registro;
    private $_pessoa;
    private $_vinculo;
    private $_tpInscricao;
    private $_nrInscricao;
    private $_nrRecolhimento;
    private $_nrCategoria;
    private $_competencias;


    public function geraRegistroInfCompl($registro, $pessoa, $vinculo){

        $this->_registro = $registro;
        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;
        $this->_geraRegistro($this->_competencias[RegistroGRRF::GRRF_COMP_INF_COMPL]);

    }

    private function _geraRegistro($competencias){

        //bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro($this->_registro, $this->_tpInscricao, $this->_nrInscricao);

        //bloco 4 | 018-032 | campo destinado a zeros.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 5 | 033-043 | pis/pasep/ci.
        $identificador = $this->_pessoa->historico->getNrpispaseppes();
        $this->atribuiDado($identificador, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 6 | 044-051 | data de admissao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        //bloco 7 | 052-053 | categoria do trabalahador.
        $this->atribuiDado($this->_nrCategoria, 2);
        
        //blocos 8 a 35 | pares competencia e valor - 14 competencias
        foreach ($competencias as $mescompetencia => $valor) {
            //bloco [par] | competencia.
            $dtmescompetenc = DateUtil::getDataDeString($mescompetencia);
            $dtmescompetenc = $dtmescompetenc->format('Y/m');
            $this->atribuiDado($dtmescompetenc, 6, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            //bloco [impar] | valor.
            $this->atribuiDado($valor, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }
        if (count($competencias) < 14) {
            //complementa o registro para completar 14 pares competencia/valor
            for ($i=0; $i<(14-count($competencias)); $i++) {
                //bloco [par] | competencia.
                $this->atribuiDado(parent::CARACTER_BRANCO, 6, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
                //bloco [impar] | valor.
                $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            }
        }

        //bloco 36 | 348-359 | preencher com brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 12, parent::CARACTER_BRANCO);

        //bloco 37 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }

    private function _atribuiDataAdmissao($dtAdmissao){

        //obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataAdmissao = DateUtil::getDataDeString($dtAdmissao);
            $dataAdmissao = $dataAdmissao->format('d/m/Y');
        } else {
            $dataAdmissao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataAdmissao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataAdmissao);

    }

    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }

    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }

    public function setNrCategoria($nrCategoria)
    {
        $this->_nrCategoria = $nrCategoria;
    }

    public function setCompentencias($competencias)
    {
        $this->_competencias = $competencias;
    }
}