<?php

namespace HCM\FPA\Services;


use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\RateioLancamentoRepository;

class RateioLancamentoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, LoteLancEventoService $loteLancEventoService,  HCMParameters $hcmParameters, RateioLancamentoRepository $rateioLancamentoRepository) {
        $this->entityManager = $entityManager;
        $this->loteLancEventoService  = $loteLancEventoService;
        $this->hcmParameters = $hcmParameters;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
    }

    public function exportaLote($itens, $nrorg, $dtmescompetenc, $nreventom)
    {   
        foreach($itens as $item) {
            $itensLote[] = array(
                'NRVINCULOM' => $item['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $nreventom,
                'NMEVENTOH' => '',
                'VRREFFOLHALOTE' => null,
                'VREVENTOLOTE' => $item['VRVALOR']
            );
        }

        $lote = $this->loteLancEventoService->geraLoteLancEvento(
            $nrorg, $dtmescompetenc, 'FPA', 'EXPORTAÇÃO DO RATEIO DE LANÇAMENTO', 'PENDENTE', '', 1,  $itensLote
        );

    }

    public function validaQntdVinculosParaRateio($qntdVinculosEstrutGerenCargo, $estrutgeren, $ocupacao)
    {
        foreach ($qntdVinculosEstrutGerenCargo as $item) {
            if ($estrutgeren == $item['NRESTRUTGEREN'] && $ocupacao == $item['NRCARGO']) {
                $qntdVinculosRateio = $item['QNTDVINCULOS']; 
                return $qntdVinculosRateio;
            }
        }
        
    }

    public function calculaRateio($vrFinalLancamento, $porcentagemEstrutGeren, $porcentagemOcupacao, $qntdVinculosRateio)
    {
        try {
            $resultado = ($vrFinalLancamento * ($porcentagemEstrutGeren/100)) * ($porcentagemOcupacao/100);
            $resultado = $resultado/$qntdVinculosRateio;
            return $resultado;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function calculaPorcentagem($nrorg, $nrrateio, $tabela)
    {
       if($tabela == 'FPA_RATEIOESTRUT') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemEstrutura($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
            $this->rateioLancamentoRepository->setPorcentagemEstrutura($nrorg, $nrrateio, $porcentagem['NRRATEIOESTRUT'],$porcentagem['PERCENT']);
           }
       } 

       if($tabela == 'FPA_RATEIOOCUP') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemOcupacao($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
               $this->rateioLancamentoRepository->setPorcentagemOcupacao($nrorg, $nrrateio, $porcentagem['NRRATEIOOCUP'],$porcentagem['PERCENT']);
           }
       }    
   }

   public function existeItensRateio($nrorg, $nrrateio)
    {
        $existeItemEstrutura =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOESTRUT');
        $existeItemLancamento =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOLANC');
        $existeItemOcupacao = $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOOCUP' );
        if($existeItemEstrutura || $existeItemLancamento || $existeItemOcupacao) {
            return true;
        }

        return false;
    }

    public function calculaLancamento($row)
    {   
        $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG'])); 
        $vrLancamento = self::aplicaAcrescimoDesconto($row, floatval($row['VRLANCAMENTO']));
        $subtotal = null;
        if($rateio->getDsoperacao1() != null) {
            $row['VRSUBTOTAL1'] = self::calculaSubTotal($vrLancamento, $rateio->getVrtaxa1()); 
            $subtotal = self::calculaVrFinalLancamento($vrLancamento, $row['VRSUBTOTAL1'], $rateio->getDsoperacao1());
            $row['VRVALSUBTOTAL1'] = $subtotal;
        } else {
            $row['VRSUBTOTAL1'] = null;
            $row['VRVALSUBTOTAL1'] = null;
        }

        if($rateio->getDsoperacao2() != null) {   
            $lancamento = !is_null($subtotal) ? $subtotal : $vrLancamento;
            $row['VRSUBTOTAL2'] = self::calculaSubTotal($lancamento, $rateio->getVrtaxa2()); 
            $subtotal = self::calculaVrFinalLancamento($lancamento, $row['VRSUBTOTAL2'], $rateio->getDsoperacao2());
            $row['VRVALSUBTOTAL2'] = $subtotal;
        } else {
            $row['VRSUBTOTAL2'] = null;
            $row['VRVALSUBTOTAL2'] = null;
        }


        if($rateio->getDsoperacao3() != null) {
            $lancamento = !is_null($subtotal) ? $subtotal : $vrLancamento;
            $row['VRSUBTOTAL3'] = self::calculaSubTotal($lancamento, $rateio->getVrtaxa3()); 
            $subtotal = self::calculaVrFinalLancamento($lancamento, $row['VRSUBTOTAL3'], $rateio->getDsoperacao3());
            $row['VRVALSUBTOTAL3'] = $subtotal;
        } else {
            $row['VRSUBTOTAL3'] = null;
            $row['VRVALSUBTOTAL3'] = null;
        }
        
        if($subtotal == null) {
            $row['VRFINAL'] = $vrLancamento;
            return;
        }

        $row['VRFINAL'] = $subtotal;
        return;
    }

    private static function calculaSubTotal($vrLancamento, $taxa)
    {   
        $resultado = ($vrLancamento * ($taxa/100));
        return $resultado;
    }

    private static function calculaVrFinalLancamento($vrLancamento, $vrsubtotal, $operação)
    {   
        if($operação == 'SOMA') {
            $resultado = $vrLancamento + $vrsubtotal;
            return $resultado;
        }

        if($operação == 'SUBTRAI') {
            $resultado = $vrLancamento - $vrsubtotal;
            return $resultado;
        }      
    }

    private static function aplicaAcrescimoDesconto($row, $vrLancamento)
    {    
        if($row['VRACRESCIMO'] != null) {
            $vrLancamento = $vrLancamento + floatval($row['VRACRESCIMO']);
        }

        if($row['VRDESCONTO'] != null) {
            $vrLancamento = $vrLancamento - floatval($row['VRDESCONTO']);
        }
        return $vrLancamento;
    }
}