<?php
namespace Planejamento\Repositories;

class Cardadia extends \Doctrine\ORM\EntityRepository
{


    CONST STATUS_N = 'N';

    /**
     * Verifica a quantidade de clientes que fizeram a refeição no dia.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtprocesso
     *
     * @return mixed
     */
    public function verificaExisteEfetivos($nrorg, $filial, $dtprocesso){
        $sql = "SELECT CA.*
                  FROM SERVICO SE, CARDADIA CA
                 WHERE CA.NRORG = :P_NRORG
                   AND CA.CDFILIAL   = :P_CDFILIAL
                   AND CA.DTCARDAPIO = :P_DTPROCESSA
                   AND SE.NRORG = CA.NRORG
                   AND SE.CDFILIAL   = CA.CDFILIAL
                   AND SE.CDSERVFILI = CA.CDSERVFILI
                   AND SE.IDSERVPADR = :P_STATUS_N
                   AND QTCOMEFETCAR IS NULL
                   AND EXISTS (SELECT NRCARDAPIO
                                 FROM RETIRADA RE
                                WHERE RE.NRORG = CA.NRORG
                                  AND RE.CDFILIAL   = CA.CDFILIAL
                                  AND RE.NRCARDAPIO = CA.NRCARDAPIO)";

        $params = array(
            "P_NRORG"      => $nrorg,
            "P_CDFILIAL"   => $filial,
            "P_DTPROCESSA" => $dtprocesso,
            "P_STATUS_N"   => self::STATUS_N,
        );


        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Planejamento\Entities\Cardadia', "CA");
        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);

        return count($result) > 0 ? $result : false;
    }

    public function get_Meta_RestoIngesta($campos){
        $sql = "SELECT  VRPEMETAREIN
                    FROM
                       METARISL
                    WHERE
                       (CDFILIAL     = :P_CDFILIAL) AND
                       (NRORG     = :P_NRORG)       AND
                       (DTREFMETRISL = :P_DTREFMETRISL)
                    ORDER BY
                       CDFILIAL,
                       DTREFMETRISL";

        $params = array(
            "P_CDFILIAL"            => $campos['FILIAL'],
            "P_NRORG"               => $campos['NRORG'],
            "P_DTREFMETRISL"       => $campos['DIA_INICIAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function get_Meta_SobraLimpa($campos){
        $sql = "SELECT
                   DTREFMETRISL,
                   VRPEMETASOLI,
                   CDFILIAL
                FROM
                   METARISL
                WHERE
                   (CDFILIAL     = :P_CDFILIAL)     AND
                   (NRORG        = :P_NRORG)        AND
                   (DTREFMETRISL = :P_DTREFMETRISL)";

        $params = array(
            "P_CDFILIAL"            => $campos['FILIAL'],
            "P_NRORG"               => $campos['NRORG'],
            "P_DTREFMETRISL"       => $campos['DIA_INICIAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function get_TotalMediaMesDia_RestoIngesta($campos){
        $sql = "SELECT
                           SUM(QTRESTINGECA) AS TOTRESTINGE,
                           (SUM(QTRESTINGECA) / SUM(QTCOMEFETRI * VRPESPORPERCRI) * 100) AS PEMEDIAPERC,
                           COUNT(QTRESTINGECA) AS CTRESTINGE
                        FROM
                           CARDADIA
                        WHERE
                           ((QTCOMEFETRI  <> 0)            AND
                           (NOT QTCOMEFETRI IS NULL))      AND
                           ((VRPESPORPERCRI <> 0)          AND
                           (NOT VRPESPORPERCRI IS NULL))   AND
                           (CDFILIAL   = :P_CDFILIAL)      AND
                           (NRORG      = :P_NRORG)         AND
                           (CDSERVFILI = :P_CDSERVFILI)    AND
                           (DTCARDAPIO BETWEEN :P_DTINIC   AND :P_DTFINA)
                        GROUP BY
                           CDFILIAL,
                           CDSERVFILI";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? reset($result) : 0;
    }

    public function get_TotalMediaMesDia_SobraLimpa($campos){
        $sql = "SELECT
                   SUM(QTSOBRLIMPCA) AS TOTSOBRLIMP,
                   (SUM(QTSOBRLIMPCA) / SUM(QTCOMEFETSL * VRPESPORPERC) * 100) AS PEMEDIAPERC,
                   COUNT(QTSOBRLIMPCA) AS CTSOBRLIMP
                FROM
                   CARDADIA
                WHERE
                   (QTCOMEFETSL  <> 0)            AND
                   (VRPESPORPERC <> 0)            AND
                   (CDFILIAL   = :P_CDFILIAL)     AND
                   (NRORG      = :P_NRORG)        AND
                   (CDSERVFILI = :P_CDSERVFILI)   AND
                   (DTCARDAPIO BETWEEN :P_DTINIC  AND :P_DTFINA)
                GROUP BY
                   CDFILIAL,
                   CDSERVFILI";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : array();
    }

    public function get_Maior_RestoIngesta($campos){
        $sql = "SELECT
                           MAX(QTRESTINGECA / (QTCOMEFETRI * VRPESPORPERCRI) * 100) AS PEMAX
                        FROM
                           CARDADIA
                        WHERE
                           ((QTCOMEFETRI  <> 0)           AND
                           (NOT QTCOMEFETRI IS NULL))     AND
                           ((VRPESPORPERCRI <> 0)         AND
                           (NOT VRPESPORPERCRI IS NULL))  AND
                           (CDFILIAL     = :P_CDFILIAL)   AND
                           (NRORG        = :P_NRORG)      AND
                           (CDSERVFILI   = :P_CDSERVFILI) AND
                           (DTCARDAPIO BETWEEN :P_DTINIC  AND :P_DTFINA)";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function get_Maior_SobraLimpa($campos){
        $sql = "SELECT
                   MAX(QTSOBRLIMPCA / (QTCOMEFETSL * VRPESPORPERC) * 100) AS PEMAX
                FROM
                   CARDADIA
                WHERE
                   (QTCOMEFETSL  <> 0)              AND
                   (VRPESPORPERC <> 0)              AND
                   (CDFILIAL     = :P_CDFILIAL)     AND
                   (NRORG        = :P_NRORG)        AND
                   (CDSERVFILI   = :P_CDSERVFILI)   AND
                   (DTCARDAPIO BETWEEN :P_DTINIC    AND :P_DTFINA)";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function get_Menor_RestoIngesta($campos){
        $sql = "SELECT MIN(QTRESTINGECA / (QTCOMEFETRI * VRPESPORPERCRI) * 100) AS PEMIN
                    FROM
                       CARDADIA
                    WHERE
                       (CDFILIAL   = :P_CDFILIAL)                              AND
                       (NRORG      = :P_NRORG)                                 AND
                       (CDSERVFILI = :P_CDSERVFILI)                            AND
                       (DTCARDAPIO BETWEEN :P_DTINIC AND :P_DTFINA)            AND
                       (QTCOMEFETRI  <> 0)                                     AND
                       (VRPESPORPERCRI <> 0)                                   AND
                       (QTRESTINGECA <> 0)";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function get_Menor_SobraLimpa($campos){
        $sql = "SELECT
                   MIN(QTSOBRLIMPCA / (QTCOMEFETSL * VRPESPORPERC) * 100) AS PEMIN
                FROM
                   CARDADIA
                WHERE
                   (QTCOMEFETSL  <> 0)            AND
                   (VRPESPORPERC <> 0)            AND
                   (QTSOBRLIMPCA <> 0)            AND
                   (CDFILIAL   = :P_CDFILIAL)     AND
                   (NRORG      = :P_NRORG)        AND
                   (CDSERVFILI = :P_CDSERVFILI)   AND
                   (DTCARDAPIO BETWEEN :P_DTINIC  AND :P_DTFINA)";

        $params = array(
            "P_CDFILIAL"   => $campos['FILIAL'],
            "P_NRORG"      => $campos['NRORG'],
            "P_CDSERVFILI" => $campos['SERVICO'],
            "P_DTINIC"     => $campos['DIA_INICIAL'],
            "P_DTFINA"     => $campos['DIA_FINAL']
        );

        $result = (float)$this->_em->getConnection()->fetchColumn($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function verificaDiferencaComensais($arredondaComensais, $cdfilial, $cdservico, $dtmes, $dataProc) {
    	$params = array (
    			"P_ARREDONDA" => $arredondaComensais,
    			"P_CDFILIAL" => $cdfilial,
    			"P_CDSERVICO" => $cdservico,
    			"P_DTMES" => '01/'.$dtmes,
    			"P_DTPROC" => $dataProc
    	);
    	$sql = "SELECT
				CDFILIAL,
				NMFILIAL,
				CDSERVFILI,
				NMSERVICO,
				DTCARDAPIO,
				CDTIPOPRAT,
				NMTIPOPRAT,
				COMENSEST,
				COMENSDIA,
				DIFERENCA
				FROM
				(SELECT
					C.CDFILIAL,
					F.NMFILIAL,
					C.CDSERVFILI,
					S.NMSERVICO,
					C.DTCARDAPIO,
					CC.CDTIPOPRAT,
					TP.NMTIPOPRAT,
					SUM(DECODE(:P_ARREDONDA,
						'S', ROUND(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR),
						CC.VRPECONSCARD / 100 * C.QTCOMESTICAR)) AS COMENSEST,
			C.QTCOMESTICAR AS COMENSDIA,
			SUM(DECODE(:P_ARREDONDA,
				'S', ROUND(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR),
				CC.VRPECONSCARD / 100 * C.QTCOMESTICAR)) - C.QTCOMESTICAR AS DIFERENCA
			FROM
			CARDADIA C,
			COMPCARD CC,
			CNFCAMES CF,
			TIPOPRAT TP,
			FILIAL F,
			SERVICO S
			WHERE
			F.CDFILIAL = :P_CDFILIAL AND
			C.CDFILIAL = F.CDFILIAL AND
			C.CDSERVFILI = :P_CDSERVICO AND
			C.DTMESCARD = :P_DTMES AND
			C.CDFILIAL = CC.CDFILIAL AND
			C.NRCARDAPIO = CC.NRCARDAPIO AND
			C.CDFILIAL = CF.CDFILIAL AND
			C.CDSERVFILI = CF.CDSERVFILI AND
			C.DTMESCARD = CF.DTREFECNFMES AND
			CC.NRSEQPRATCAR = CF.NRSEQCNFMES AND
			CC.CDTIPOPRAT = CF.CDTIPOPRAT AND
			CF.IDOPCCNFMES <> 'S' AND
			CF.IDCONCNFMES = 'S' AND
			CC.CDTIPOPRAT = TP.CDTIPOPRAT AND
			C.CDFILIAL = S.CDFILIAL AND
			C.CDSERVFILI = S.CDSERVFILI AND
			C.DTCARDAPIO >= :P_DTPROC
			GROUP BY
			C.CDFILIAL,
			F.NMFILIAL,
			C.CDSERVFILI,
			S.NMSERVICO,
			C.DTCARDAPIO,
			C.QTCOMESTICAR,
			CC.CDTIPOPRAT,
			TP.NMTIPOPRAT
			HAVING
			ROUND(SUM(DECODE(:P_ARREDONDA,
				'S', ROUND(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR),
				CC.VRPECONSCARD / 100 * C.QTCOMESTICAR)), 3) > ROUND(C.QTCOMESTICAR, 3))";
    
    	$eManager = $this->getEntityManager();
    	$result = $eManager->getConnection()->fetchAll($sql,$params);
    	return $result;
    } 
    

    public function consultaRetiradasCardapioRelacLancto($nrLancto, $filial, $nrorg){
        $sql = "SELECT AR.CDFILIAL, AR.NRLANCESTQ, AR.CDFILIALRET, AR.NRLANCESTRET,
                       RE.NRRETIRADA, CD.NRCARDAPIO, CD.CDSERVFILI, DTRETIRADA
                  FROM CARDADIA CD, RETIRADA RE, AJURELACRET AR
                 WHERE CD.NRCARDAPIO   = RE.NRCARDAPIO
                   AND CD.CDFILIAL     = RE.CDFILIAL
                   AND CD.NRORG        = RE.NRORG
                   AND RE.NRLANCESTQ   = AR.NRLANCESTRET
                   AND RE.CDFILANCESTQ = AR.CDFILIAL
                   AND RE.NRORG        = AR.NRORG
                   AND AR.CDFILIALRET  = :P_CDFILIAL
                   AND AR.NRORG        = :P_NRORG
                   AND AR.NRLANCESTQ   = :P_NRLANCESTQ
                   AND AR.CDFILIAL     = :P_CDFILIAL";

        $params = array(
            'P_NRLANCESTQ' => $nrLancto,
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        return (count($result) > 0) ? current($result) : false;
    }

    public function deletaCardapioNoPeriodo($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal) {
        $conn = $this->getEntityManager()->getConnection();

        $query =
            "
            DELETE CARDADIA
            WHERE NRORG = ?
              AND CDFILIAL = ?
              AND CDSERVFILI = ?
              AND DTCARDAPIO BETWEEN ? AND ?
            "
        ;

        $result = $conn->executeQuery($query, array($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal));
        return (bool)$result;
    }

    public function deletaCardapiosSemPrato($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal) {
        $conn = $this->getEntityManager()->getConnection();

        $query =
            "
            DELETE CARDADIA CA
            WHERE QTCOMESTICAR > 0
                AND NRORG = ?
                AND CDFILIAL  = ?
                AND CDSERVFILI = ?
                AND DTCARDAPIO BETWEEN ? AND ?
                AND NOT EXISTS(SELECT CO.CDFILIAL
                               FROM COMPCARD CO
                               WHERE CO.CDFILIAL = CA.CDFILIAL
                                 AND CO.NRCARDAPIO = CA.NRCARDAPIO
                                 AND CO.NRORG = CA.NRORG)
            "
        ;

        $result = $conn->executeQuery($query, array($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal));
        return (bool)$result;
    }

    public function temCardapioNoMes($nrorg, $cdfilial, $cdservfili, $datainicial, $datafinal) {
        $conn = \Setforms2\Shared\Connection::getInstance();

        $query = 'SELECT * FROM CARDADIA WHERE NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ? AND DTCARDAPIO BETWEEN ? AND ?';

        return $conn->fetchAll($query, array($nrorg, $cdfilial, $cdservfili, $datainicial, $datafinal));
    }

    public function temPratoNosCardapiosDoMes($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal) {
        $conn = \Setforms2\Shared\Connection::getInstance();
        $query = "
            SELECT *
                FROM COMPCARD CO, CARDADIA CA
            WHERE CA.CDFILIAL  = ?
                AND CA.CDSERVFILI = ?
                AND CA.DTCARDAPIO BETWEEN ? AND ?
                AND CO.CDFILIAL = CA.CDFILIAL
                AND CO.NRCARDAPIO = CA.NRCARDAPIO
        ";
        return $conn->fetchAll($query, array($nrorg, $cdfilial, $cdservfili, $dtinicial, $dtfinal));
    }

    public static function verificaPratoCardapio($nrorg, $filial, $cdservfili, $dtinicial, $dtfinal, $cdprato, $cdfiliaux){
        $sql = "SELECT CC.CDPRATO
                  FROM CARDADIA C, COMPCARD CC
                 WHERE (INSTR(:P_CDFILIAL, C.CDFILIAL) > 0)
                   AND (C.NRORG = :P_NRORG)
                   AND (INSTR(:P_CDSERVFILI, C.CDSERVFILI) > 0)
                   AND C.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                   AND (INSTR(CC.CDFILIAL, C.CDFILIAL) > 0)
                   AND C.NRCARDAPIO = CC.NRCARDAPIO
                   AND CC.CDPRATO = :P_CDPRATO
                   AND CC.CDFILAUXPRAT = :P_CDFILAUX";

        $param = array(
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_CDSERVFILI' => $cdservfili,
            'P_DTINICIAL' => $dtinicial,
            'P_DTFINAL' => $dtfinal,
            'P_CDPRATO' => $cdprato,
            'P_CDFILAUX' => $cdfiliaux,
        );

        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $param);
        return count($result)>0 ? $result : array();
    }

    public static function verificaCardapioSemPrato($nrorg, $filial, $cdservfili, $dtinicial, $dtfinal){
        $sql = "SELECT DTCARDAPIO, QTCOMESTICAR FROM CARDADIA CA
                WHERE NRORG = :P_NRORG
                    AND CA.QTCOMESTICAR > 0
   					AND CDFILIAL  = :P_CDFILIAL
                    AND CDSERVFILI = :P_CDSERVFILI
                    AND DTCARDAPIO BETWEEN :P_DTINICIAL and :P_DTFINAL
                    AND NOT EXISTS(SELECT CO.CDFILIAL
                                   FROM COMPCARD CO
                                   WHERE CO.CDFILIAL = CA.CDFILIAL
                                     AND CO.NRCARDAPIO = CA.NRCARDAPIO
                                     AND CO.NRORG = CA.NRORG)";

        $param = array(
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_CDSERVFILI' => $cdservfili,
            'P_DTINICIAL' => $dtinicial,
            'P_DTFINAL' => $dtfinal,
        );

        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $param);
        return count($result) > 0 ? $result : false;
    }


    public function getNrcardapio($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT NRCARDAPIO
                  FROM CARDADIA CA, LISTA LS
                WHERE CA.NRORG      = :P_NRORG
                  AND CA.NRORG      = LS.NRORG
                  AND LS.NRLISTA    = :P_NRLISTASERV
                  AND CA.CDFILIAL   = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioLivre($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = PF.NRORG
                  AND CA.NRORG = PG.NRORG
                  AND CA.NRORG = LS.NRORG
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND PF.CDFILIAL = CA.CDFILIAL
                  AND CA.DTCARDAPIO >= TRUNC(DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0))
                  AND NOT EXISTS(SELECT RE.CDFILIAL
                                 FROM RETIRADA RE
                                 WHERE RE.NRORG = :P_NRORG
                                   AND RE.CDFILIAL = CA.CDFILIAL
                                   AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                  AND NOT EXISTS(SELECT EF.CDFILIAL
                                 FROM EFETCLIE EF
                                 WHERE EF.NRORG = :P_NRORG
                                   AND EF.CDFILIAL   = CA.CDFILIAL
                                   AND EF.CDSERVFILI = CA.CDSERVFILI
                                   AND EF.DTEFETCLIE = CA.DTCARDAPIO
                                   AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioBloqLivre($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                AND CA.NRORG = LS.NRORG
                AND LS.NRLISTA = :P_NRLISTASERV
                AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                AND CDDIETPAD IS NULL
                AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL

                MINUS

                SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                AND CA.NRORG = PF.NRORG
                AND CA.NRORG = PG.NRORG
                AND CA.NRORG = LS.NRORG
                AND LS.NRLISTA = :P_NRLISTASERV
                AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                AND CDDIETPAD IS NULL
                AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                AND PF.CDFILIAL = CA.CDFILIAL
                AND CA.DTCARDAPIO >= DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0)
                AND NOT EXISTS(SELECT RE.CDFILIAL
                FROM RETIRADA RE
                WHERE RE.NRORG = :P_NRORG
                AND RE.CDFILIAL = CA.CDFILIAL
                AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                AND NOT EXISTS(SELECT EF.CDFILIAL
                FROM EFETCLIE EF
                WHERE EF.NRORG = :P_NRORG
                AND EF.CDFILIAL   = CA.CDFILIAL
                AND EF.CDSERVFILI = CA.CDSERVFILI
                AND EF.DTEFETCLIE = CA.DTCARDAPIO
                AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioPadronizado($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, CARDAMES CM, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = PF.NRORG
                  AND CA.NRORG = PG.NRORG
                  AND CA.NRORG = CM.NRORG(+)
                  AND CA.NRORG = LS.NRORG
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND PF.CDFILIAL = CA.CDFILIAL
                  AND CA.CDFILIAL = CM.CDFILIAL(+)
                  AND CA.CDSERVFILI = CM.CDSERVFILI(+)
                  AND CA.DTMESCARD = CM.DTMESCARDAPIO(+)
                  AND ((CM.IDCARDLIBAGRE <> 'S') OR (CM.IDCARDLIBAGRE IS NULL))
                  AND CA.DTCARDAPIO >= TRUNC(DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0))
                  AND NOT EXISTS(SELECT RE.CDFILIAL
                                 FROM RETIRADA RE
                                 WHERE RE.NRORG = :P_NRORG
                                   AND RE.CDFILIAL = CA.CDFILIAL
                                   AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                  AND NOT EXISTS(SELECT EF.CDFILIAL
                                 FROM EFETCLIE EF
                                 WHERE EF.NRORG = :P_NRORG
                                   AND EF.CDFILIAL   = CA.CDFILIAL
                                   AND EF.CDSERVFILI = CA.CDSERVFILI
                                   AND EF.DTEFETCLIE = CA.DTCARDAPIO
                                   AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioBloqPadronizado($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = LS.NRORG
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL

                MINUS

                SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, CARDAMES CM, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = PF.NRORG
                  AND CA.NRORG = PG.NRORG
                  AND CA.NRORG = CM.NRORG(+)
                  AND CA.NRORG = LS.NRORG
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND PF.CDFILIAL = CA.CDFILIAL
                  AND CA.CDFILIAL = CM.CDFILIAL(+)
                  AND CA.CDSERVFILI = CM.CDSERVFILI(+)
                  AND CA.DTMESCARD = CM.DTMESCARDAPIO(+)
                  AND ((CM.IDCARDLIBAGRE <> 'S') OR (CM.IDCARDLIBAGRE IS NULL))
                  AND CA.DTCARDAPIO >= DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0)
                  AND NOT EXISTS(SELECT RE.CDFILIAL
                                 FROM RETIRADA RE
                                 WHERE RE.NRORG = :P_NRORG
                                   AND RE.CDFILIAL = CA.CDFILIAL
                                   AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                  AND NOT EXISTS(SELECT EF.CDFILIAL
                                 FROM EFETCLIE EF
                                 WHERE EF.NRORG = :P_NRORG
                                   AND EF.CDFILIAL   = CA.CDFILIAL
                                   AND EF.CDSERVFILI = CA.CDSERVFILI
                                   AND EF.DTEFETCLIE = CA.DTCARDAPIO
                                   AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioDescentralizado($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, LISTA LS, CTRLAPROVCARD CT
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = PF.NRORG
                  AND CA.NRORG = PG.NRORG
                  AND CA.NRORG = LS.NRORG
                  AND CA.NRORG = CT.NRORG(+)
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND PF.CDFILIAL = CA.CDFILIAL
                  AND CT.CDFILIAL(+) = CA.CDFILIAL
                  AND CT.CDSERVFILI(+) = CA.CDSERVFILI
                  AND CT.DTMESCARDAPROV(+) = CA.DTMESCARD
                  AND ((CT.DTLIBECARDMATR IS NULL) OR NOT(CT.DTAPROCARDFILI IS NULL))
                  AND CA.DTCARDAPIO >= TRUNC(DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0))
                  AND NOT EXISTS(SELECT RE.CDFILIAL
                                 FROM RETIRADA RE
                                 WHERE RE.NRORG = :P_NRORG
                                   AND RE.CDFILIAL = CA.CDFILIAL
                                   AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                  AND NOT EXISTS(SELECT EF.CDFILIAL
                                 FROM EFETCLIE EF
                                 WHERE EF.NRORG = :P_NRORG
                                   AND EF.CDFILIAL   = CA.CDFILIAL
                                   AND EF.CDSERVFILI = CA.CDSERVFILI
                                   AND EF.DTEFETCLIE = CA.DTCARDAPIO
                                   AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public function getCardapioBloqDescentralizado($nrorg, $nrlista, $dtinicial, $dtfinal)
    {
        $sql
            = "SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, LISTA LS
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = LS.NRORG
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL

                MINUS

                SELECT CA.CDFILIAL, CA.NRCARDAPIO, CA.CDSERVFILI, SYSDATE, CA.DTCARDAPIO
                FROM CARDADIA CA, PARAMFILIAL PF, PARAMGERAL PG, LISTA LS, CTRLAPROVCARD CT
                WHERE CA.NRORG = :P_NRORG
                  AND CA.NRORG = PF.NRORG
                  AND CA.NRORG = PG.NRORG
                  AND CA.NRORG = LS.NRORG
                  AND CA.NRORG = CT.NRORG(+)
                  AND LS.NRLISTA = :P_NRLISTASERV
                  AND CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4)
                  AND CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5)
                  AND CDDIETPAD IS NULL
                  AND CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND PF.CDFILIAL = CA.CDFILIAL
                  AND CT.CDFILIAL(+) = CA.CDFILIAL
                  AND CT.CDSERVFILI(+) = CA.CDSERVFILI
                  AND CT.DTMESCARDAPROV(+) = CA.DTMESCARD
                  AND ((CT.DTLIBECARDMATR IS NULL) OR NOT(CT.DTAPROCARDFILI IS NULL))
                  AND CA.DTCARDAPIO >= TRUNC(DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, PF.DTPROCESSA) + NVL(PG.QTDIABLOQCAR, 0))
                  AND NOT EXISTS(SELECT RE.CDFILIAL
                                 FROM RETIRADA RE
                                 WHERE RE.NRORG = :P_NRORG
                                   AND RE.CDFILIAL = CA.CDFILIAL
                                   AND RE.NRCARDAPIO = CA.NRCARDAPIO)
                  AND NOT EXISTS(SELECT EF.CDFILIAL
                                 FROM EFETCLIE EF
                                 WHERE EF.NRORG = :P_NRORG
                                   AND EF.CDFILIAL   = CA.CDFILIAL
                                   AND EF.CDSERVFILI = CA.CDSERVFILI
                                   AND EF.DTEFETCLIE = CA.DTCARDAPIO
                                   AND (EF.CDFILIFATU IS NOT NULL AND EF.NRAPURACFAT IS NOT NULL))";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista,
            'P_DTINICIAL'   => $dtinicial,
            'P_DTFINAL'     => $dtfinal
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    public static function geraNrcardapio($cdfilial, $cdservfili, $dtcardapio, $nrorg){
        $sql =<<<SQL
        SELECT NRCARDAPIO
FROM CARDADIA
WHERE CDFILIAL =:CDFILIAL and
  CDSERVFILI   =:CDSERVFILI and
  DTCARDAPIO   =:DTCARDAPIO and
  NRORG        =:NRORG

SQL;
        $params = array(
            'CDFILIAL'=>$cdfilial,
            'CDSERVFILI'=>$cdservfili,
            'DTCARDAPIO'=>$dtcardapio,
            'NRORG'=>$nrorg);


        try{
            $conn = \Setforms2\Shared\Registry::getEntityManager();
            $conn = $conn->getConnection();
            return $conn-> fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;

        }
        $sql = $sql[0];
        if($sql){
            $nrcardapio = $sql['NRCARDAPIO'];
        } else{
            $nrcardapio = false;
        }
        return $nrcardapio;
    }
    public static function buscaQtdComensaisCardapio($cdfilial,$data,$nrorg,$nrcardapio){
        $sql =<<<SQL
        SELECT QTCOMESTICAR
FROM CARDADIA
WHERE DTCARDAPIO =:DTCARDAPIO
AND CDFILIAL     =:CDFILIAL
AND NRCARDAPIO   =:NRCARDAPIO
AND NRORG        =:NRORG
SQL;
        $params = array(

            'DTCARDAPIO'=>$data,
            'CDFILIAL'=>$cdfilial,
            'NRCARDAPIO'=>$nrcardapio,
            'NRORG'=>$nrorg);


        try{
            $conn = \Setforms2\Shared\Registry::getEntityManager();
            $conn = $conn->getConnection();
            return $conn-> fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }

    }


    public static function retornaCardapioSemana($qtcomens,$cdfilial,$cdservfili,$dtinicial,$dtfinal,$cddieta,$nrorg){

        $sql = "SELECT
                   CA.CDFILIAL,
                   CA.CDSERVFILI,
                   SE.NMSERVICO,
                   SE.NMFANTASSERV,
                   CA.DTCARDAPIO,
                   CM.CDTIPOPRAT,
                   TP.NMTIPOPRAT,
                   TO_CHAR(CA.DTCARDAPIO,'D') DIASEMANA,
                   CONCAT(ROUND(DECODE(:P_QTCOMENS, 'T', ((CA.QTCOMESTICAR * CC.VRPECONSCARD) / 100), CC.VRPECONSCARD)),' '||PR.NMPRATO) NMPRATO,
                   CC.CDPRATO,
                   CC.CDFILAUXPRAT,
                   CC.NRSEQPRATCAR,
                   CA.CDDIETPAD,
                   DI.DSDIETPAD,
                   CC.NRORDPRATCAR,
                   CA.DTMESCARD
                FROM
                   PRATO PR,
                   TIPOPRAT TP,
                   SERVICO SE,
                   CNFCAMES CM LEFT JOIN COMPCARD CC
                    ON (CC.NRSEQPRATCAR = CM.NRSEQCNFMES)
                    AND (CC.CDTIPOPRAT = CM.CDTIPOPRAT)
                    AND (CC.NRORG = CM.NRORG),
                   CARDADIA CA LEFT JOIN DIETPADR DI
                    ON (CA.CDDIETPAD  = DI.CDDIETPAD)
                    AND (CA.NRORG = DI.NRORG)
                WHERE
                   (SE.CDFILIAL         = :P_CDFILIAL)                              AND
                   (INSTR(:P_CDSERVFILI,SE.CDSERVFILI) <> 0 OR :P_CDSERVFILI = 'A') AND
                   (CA.CDFILIAL         = SE.CDFILIAL)                              AND
                   (CA.CDSERVFILI       = SE.CDSERVFILI)                            AND
                   (CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL)              AND
                   (INSTR(:P_CDDIETA, DI.CDDIETPAD) <> 0 OR :P_CDDIETA = 'T')       AND
                   (CM.CDFILIAL         = CA.CDFILIAL)                              AND
                   (CM.CDSERVFILI       = CA.CDSERVFILI)                            AND
                   (CM.DTREFECNFMES     = CA.DTMESCARD)                             AND
                   (CM.IDIMPCNFMES      = 'S')                                      AND
                   (CC.CDFILIAL         = CA.CDFILIAL)                              AND
                   (CC.NRCARDAPIO       = CA.NRCARDAPIO)                            AND
                   (PR.CDFILAUXPRAT     = CC.CDFILAUXPRAT)                          AND
                   (PR.CDPRATO          = CC.CDPRATO)                               AND
                   (TP.CDTIPOPRAT       = CM.CDTIPOPRAT)
                ORDER BY
                   CA.DTCARDAPIO,
                   CA.CDSERVFILI";

        $params = array(
            'P_QTCOMENS'=>$qtcomens,
            'P_CDFILIAL'=>$cdfilial,
            'P_DTINICIAL'=>$dtinicial,
            'P_DTFINAL'=>$dtfinal,
            'P_CDDIETA'=>$cddieta,
            'P_CDSERVFILI'=>$cdservfili,
            'P_NRORG'=>$nrorg);

        try{
            $conn = \Setforms2\Shared\Registry::getEntityManager();
            $conn = $conn->getConnection();
            return $conn->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}